"""
__init__ de mi paquete
"""
from myWebKraken_package.myWebKraken import myAppKraken
from myWebKraken_package.myKrakenAPI import obtenerDatosKraken
from myWebKraken_package.myKrakenAPI import intervalosValidos
from myWebKraken_package.myKrakenAPI import incorporaMetricasMoviles
from myWebKraken_package.myKrakenAPI import rsi
from myWebKraken_package.myKrakenAPI import obtenerPares
from myWebKraken_package.myKrakenAPI import convierteFechaUnix
from myWebKraken_package.myKrakenAPI import convierteFechaHoraUnix



import dash
from dash import dcc, html, Input, Output
from myWebKraken_package import myKrakenAPI
from plotly.subplots import make_subplots
import plotly.graph_objects as go

class myAppKraken(dash.Dash):

    def __init__(self):
        """
        Esta funcion inicializa la App.
        """

        #Definición de estilo
        external_stylesheets = [
            {
                "href": "https://fonts.googleapis.com/css2?"
                        "family=Lato:wght@400;700&display=swap",
                "rel": "stylesheet",
            },
        ]

        #Periodos usados para la media movil y RSI
        periodos = 5

        super().__init__(#__name__,
            external_stylesheets=external_stylesheets)

        #Establecemos Titulo y elementos de la hoja
        self.title = "Análisis de la evolución de KriptoAssets (Fuente Kraken)"
        self.layout = html.Div(
            children=[
                # Cabecera de la Web
                html.Div(
                    children=[
                        html.P(children="🐣", className="header-emoji"),
                        html.H1(
                            children="Análisis de la evolución de KriptoAssets ", className="header-title"
                        ),
                        html.P(
                            children="(Fuente Kraken)",
                            className="header-description",
                        ),
                        html.P(
                            children="by LADM",
                            className="header-description",
                        ),
                    ],
                    className="header",
                ),
                # Dos filtros para elegir el kriptoactivo y el intervalo de muestreo
                html.Div(
                    children=[
                        #Filtro de kripto activo obteniendo el listado de Kraken
                        html.Div(
                            children=[
                                html.Div(children="KriptoActivo", className="menu-title"),
                                dcc.Dropdown(
                                    id="filtro-KriptoActivo",
                                    options=[
                                        {"label": KriptoActivo, "value": KriptoActivo}
                                        for KriptoActivo in myKrakenAPI.obtenerPares()
                                    ],
                                    value="ETHUSDT",
                                    clearable=False,
                                    className="dropdown",
                                ),
                            ]
                        ),
                        # Filtro de intervalo obteniendo el listado de myKrakenAPI
                        html.Div(
                            children=[
                                html.Div(children="Intervalo", className="menu-title"),
                                dcc.Dropdown(
                                    id="filtro-intervalo",
                                    options=[
                                        {"label": intervalo, "value": intervalo}
                                        for intervalo in myKrakenAPI.intervalosValidos.keys()
                                    ],
                                    value="1 Minuto",
                                    clearable=False,
                                    searchable=False,
                                    className="dropdown",
                                ),
                            ],
                        ),
                    ],
                    className="menu",
                ),
                # Cabecera de los graficos
                html.H1(children="Analisis del precio del activo", ),
                html.P(
                    children="Analizar el comportamiento del precio del activo"
                             " su media movil "
                             " y RSI",
                ),
                # Los dos graficos que se refrescaran con callback
                html.Div(
                    children=[
                        html.Div(
                            children=dcc.Graph(
                                id="grafica-precio", config={"displayModeBar": False},
                            ),
                            className="card",
                        ),
                        html.Div(
                            children=dcc.Graph(
                                id="grafica-rsi", config={"displayModeBar": False},
                            ),
                            className="card",
                        ),
                    ],
                    className="wrapper",
                ),
            ]
        )

        # Callback para refescar los graficos
        @self.callback(
            [Output("grafica-precio", "figure"), Output("grafica-rsi", "figure")],
            [
                Input("filtro-KriptoActivo", "value"),
                Input("filtro-intervalo", "value"),
            ],
        )
        def update_charts(KriptoActivo, intervalo):
            """
            Esta funcion accede al API de Kraken
            descarga el dataframe con la cotizaacion
            de los kriptoactivos con la frecuencia de muestreo
            y devuelve los dos graficos, de precios/media movil y RSI.

            Args:
                KriptoActivo: Par de monedas de que queremos el precio.
                intervalo: intervalo de muestreo

            Returns:
                Graficos de precios/media movil y RSI.
            """

            filtered_data = myKrakenAPI.obtenerDatosKraken(KriptoActivo, myKrakenAPI.intervalosValidos[intervalo])
            if filtered_data is not None:
                filtered_data = myKrakenAPI.incorporaMetricasMoviles(filtered_data, "close", periodos)

            #Grafico de RSI
            rsi_chart_figure = {
                "data": [
                    {
                        "x": filtered_data.index,
                        "y": filtered_data["RSI"],
                        "type": "lines",
                    },
                ],
                "layout": {
                    "title": {
                        "text": "RSI",
                        "x": 0.05,
                        "xanchor": "left"
                    },
                    "xaxis": {"fixedrange": True},
                    "yaxis": {"fixedrange": True},
                    "colorway": ["#E12D39"],
                },
            }

            # Grafico de Precio y Media Movil
            price_chart_figure = make_subplots()
            price_chart_figure.add_trace(go.Scatter(x=filtered_data.index, y=filtered_data.close,
                                                    mode='lines',
                                                    name='Close'),
                                         secondary_y=False)
            price_chart_figure.add_trace(go.Scatter(x=filtered_data.index, y=filtered_data.RollingMean,
                                                    mode='lines',
                                                    name='RollingMean'),
                                         secondary_y=False)
            price_chart_figure.update_layout(
                title_text="Evolución del precio de " + KriptoActivo
            )

            return price_chart_figure, rsi_chart_figure


    def lanzaServidor(self):
        """
        Metodo para lanzar el servidor
        """
        try:
            self.run_server(debug=True)
        except Exception as err:
            print("Error en llamada a mAppKraken:", err)




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['myWebKraken_package']

package_data = \
{'': ['*']}

install_requires = \
['dash>=2.6.2,<3.0.0', 'pykrakenapi>=0.3.1,<0.4.0']

setup_kwargs = {
    'name': 'mywebkraken-package',
    'version': '0.1.4',
    'description': '',
    'long_description': '# myWebKraken\nProyecto prueba que se conecta a Kraken,  descarga precios de kripro activos y los uestra en web\n\nPAra instalar:\n```\npip install -r requirements.txt\n```\n\nPara ejecutar\n\n```\nmain.py\n```',
    'author': 'LADM',
    'author_email': 'you@example.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import krakenex
import pykrakenapi
import datetime
import time

intervalosValidos = {
    "1 Minuto": 1,
    "5 Minutos": 5,
    "15 Minutos": 15,
    "30 Minutos": 30,
    "1 Hora": 60,
    "4 Horas": 240,
    "1 Dia": 1440,
    "1 Semana": 10080,
    "15 Días": 21600
}


def obtenerDatosKraken(par_monedas, intervalo, inicio=None):
    """
    Esta funcion accede al API de Kraken
    Descarga la maxima historia disponible de precios
    del PAR_MONEDAS e intervalo elegido.

    Args:
        par_monedas: Par de monedas de que queremos el precio.
        intervalo: intervalo de tiempo entre precios

    Returns:
        DataFrame (pandas) con los precios de la kriptomoneda.
    """

    #Comprobamos que el intervalo es valido
    if intervalo not in intervalosValidos.values():
        print("Intervalo no Valido")
        return None
    #Accedemos al API Kraken y devolvemos el DF
    try:
        api = krakenex.API()
        k = pykrakenapi.KrakenAPI(api)
        ohlc, last = k.get_ohlc_data(par_monedas, interval=intervalo, since=inicio)
        return ohlc
    except Exception as err:
        print("Error en la descarga:", err)
        return None

#Fuente https://www.roelpeters.be/many-ways-to-calculate-the-rsi-in-python-pandas/
def rsi(df, Columnaprecio, periods=14, ema=True):
    """
    Returns a pd.Series with the relative strength index.
    """
    try:
        close_delta = df[Columnaprecio].diff()

        # Make two series: one for lower closes and one for higher closes
        up = close_delta.clip(lower=0)
        down = -1 * close_delta.clip(upper=0)

        if ema == True:
            # Use exponential moving average
            ma_up = up.ewm(com=periods - 1, adjust=True, min_periods=periods).mean()
            ma_down = down.ewm(com=periods - 1, adjust=True, min_periods=periods).mean()
        else:
            # Use simple moving average
            ma_up = up.rolling(window=periods, adjust=False).mean()
            ma_down = down.rolling(window=periods, adjust=False).mean()

        rsi = ma_up / ma_down
        rsi = 100 - (100 / (1 + rsi))
        return rsi
    except Exception as err:
        print("Error en Funcion RSI:", err)
        return None


def incorporaMetricasMoviles(df, columna, periodos):
    """
    Esta funcion incorpora la media movil y el RSI de
    una columna usando periodos como ventana.

    Args:
        df: DataFrame.
        columna: la columna a promediar
        periodos: la venta a a promediar

    Returns:
        DataFrame (pandas) con las nuevas columnas.
    """
    try:
        df['RollingMean'] = df[columna].rolling(periodos).mean()
        df["RSI"] = rsi(df, columna, periodos)
        return df
    except Exception as err:
        print("Error en Funcion incorporaMetricasMoviles:", err)
        return None


def obtenerPares():
    """
    Esta funcion devuelve los pares de divisa
    disponibles en Kraken.

    Returns:
        Listado de Pares de Divisas.
    """
    try:
        api = krakenex.API()
        k = pykrakenapi.KrakenAPI(api)
        return k.get_tradable_asset_pairs().altname
    except Exception as err:
        print("Error en Funcion obtenerPares:", err)
        return None


def convierteFechaUnix(year, month, day):
    """
    Esta devuelve el tiempo Unix de una fecha

    Args:
        year: El Año.
        month: El Mes
        day: El dia

    Returns:
        Tiempo Unix
    """
    try:
        date_time = datetime.datetime(year, month, day)
        fechaUnix = int(time.mktime(date_time.timetuple()))
        print("Given Date:", date_time)
        print("UNIX timestamp:", fechaUnix)
        return fechaUnix
    except Exception as err:
        print("Error en Funcion convierteFechaUnix:", err)
        return None


def convierteFechaHoraUnix(year, month, day,hour, minute, second):
    """
    Esta devuelve el tiempo Unix de una fecha y hora

    Args:
        year: El Año.
        month: El Mes
        day: El dia
        hour: La hora
        minute: El minuto
        second: El segundo

    Returns:
        Tiempo Unix
    """
    try:
        date_time = datetime.datetime(year, month, day,hour, minute, second)
        fechaUnix = int(time.mktime(date_time.timetuple()))
        print("Given Date:", date_time)
        print("UNIX timestamp:", fechaUnix)
        return fechaUnix
    except Exception as err:
        print("Error en Funcion convierteFechaHoraUnix:", err)
        return None